<?php
/**
 * Plugin Name: PW WooCommerce On Sale! Pro
 * Plugin URI: https://www.pimwick.com/pw-woocommerce-on-sale/
 * Description: Simply the FASTEST way to schedule sales in WooCommerce!
 * Version: 1.40
 * Author: Pimwick, LLC
 * Author URI: https://www.pimwick.com
 * WC requires at least: 3.0.0
 * WC tested up to: 6.2
*/
define( 'PW_ON_SALE_VERSION', '1.40' );

defined( 'ABSPATH' ) or exit;
defined( 'PW_ON_SALE_REQUIRES_PRIVILEGE' ) or define( 'PW_ON_SALE_REQUIRES_PRIVILEGE', 'manage_woocommerce' );

if ( ! class_exists( 'PW_On_Sale' ) ) :

final class PW_On_Sale {

    private $active_sales_cache = null;

    function __construct() {
        add_action( 'init', array( 'PW_On_Sale', 'register_post_types' ), 9 );
        add_action( 'plugins_loaded', array( $this, 'plugins_loaded' ) );
        add_action( 'woocommerce_init', array( $this, 'woocommerce_init' ) );
    }

    function plugins_loaded() {
        load_plugin_textdomain( 'pimwick', false, basename( dirname( __FILE__ ) ) . '/languages' );
    }

    function woocommerce_init() {
        if ( is_admin() ) {
            require_once( 'includes/class-pimwick-license-manager.php' );
            $this->license = new Pimwick_License_Manager( __FILE__ );

            add_action( 'admin_menu', array( $this, 'admin_menu' ) );
            add_action( 'admin_enqueue_scripts', array( $this, 'admin_enqueue_scripts' ) );
            add_action( 'wp_ajax_pw-on-sale-activation', array( $this, 'ajax_activation' ) );
            add_action( 'wp_ajax_pw-on-sale-check-license', array( $this, 'ajax_check_license' ) );
            add_action( 'wp_ajax_pw-on-sale-save', array( $this, 'ajax_save_sale' ) );
            add_action( 'wp_ajax_pw-on-sale-delete', array( $this, 'ajax_sale_delete' ) );
        } else {
            if ( $this->wc_min_version( '3.0' ) ) {
                $this->price_override_hooks[] = 'woocommerce_product_get_price';
                $this->price_override_hooks[] = 'woocommerce_product_get_sale_price';
                $this->price_override_hooks[] = 'woocommerce_product_variation_get_price';
                $this->price_override_hooks[] = 'woocommerce_product_variation_get_sale_price';
            } else {
                $this->price_override_hooks[] = 'woocommerce_get_price';
                $this->price_override_hooks[] = 'woocommerce_get_sale_price';
                $this->price_override_hooks[] = 'woocommerce_get_variation_price';
                $this->price_override_hooks[] = 'woocommerce_get_variation_sale_price';
            }

            $this->hook_price_overrides();
        }

        add_filter( 'woocommerce_product_is_on_sale', array( $this, 'woocommerce_product_is_on_sale' ), 10, 2 );
        add_filter( 'woocommerce_variation_prices', array( $this, 'woocommerce_variation_prices' ), 10, 2 );
        add_filter( 'pwos_to_default_currency', array( $this, 'pwos_to_default_currency' ) );

        // Handle Flatsome's bubble cache.
        if ( function_exists( 'flatsome_percentage_get_cache' ) ) {
            add_filter( 'get_post_metadata', array( $this, 'get_post_metadata' ), 10, 4 );
        }
    }

    function hook_price_overrides() {
        foreach ( $this->price_override_hooks as $hook ) {
            add_filter( $hook, array( $this, 'override_price' ), 10, 2 );
        }
    }

    function unhook_price_overrides() {
        foreach ( $this->price_override_hooks as $hook ) {
            remove_filter( $hook, array( $this, 'override_price' ), 10, 2 );
        }
    }

    function get_active_sales() {
        if ( is_null( $this->active_sales_cache ) ) {
            $this->active_sales_cache = array();

            $all_sales = get_posts( array(
                'posts_per_page' => -1,
                'post_type' => 'pw_on_sale',
                'post_status' => 'publish'
            ) );

            foreach ( $all_sales as $sale ) {

                // Before parsing the begin/end dates and times, switch over to their local timezone.
                $configured_timezone = wc_timezone_string();
                if ( !empty( $configured_timezone ) ) {
                    $original_timezone = date_default_timezone_get();
                    date_default_timezone_set( $configured_timezone );
                }

                $sale->begin_datetime = strtotime( $sale->begin_date . ' ' . $sale->begin_time );
                $sale->end_datetime = strtotime( $sale->end_date . ' ' . $sale->end_time );

                if ( time() < $sale->begin_datetime || time() > $sale->end_datetime ) {
                    continue;
                }

                // Now that we're done formatting, switch it back.
                if ( isset( $original_timezone ) ) {
                    date_default_timezone_set( $original_timezone );
                }

                $this->active_sales_cache[] = $sale;
            }
        }

        return $this->active_sales_cache;
    }

    function override_price( $value, $product ) {

        $this->unhook_price_overrides();

        $sales = $this->get_product_sales( $product );
        foreach ( $sales as $sale ) {
            $addon_cost = 0;

            // Patch for "WooCommerce Better Product Add-ons" that add the cost after.
            if ( class_exists( 'WC_Product_Addons' ) && isset( WC()->cart ) && !is_null( WC()->cart ) ) {
                foreach ( WC()->cart->get_cart() as $cart_item ) {
                    if ( $cart_item['data'] === $product && isset( $cart_item['addons'] ) && is_array( $cart_item['addons'] ) ) {
                        foreach ( $cart_item['addons'] as $addon ) {
                            if ( $addon['price'] > 0 ) {
                                $addon_cost += $addon['price'];

                                if ( isset( $cart_item['addons_price_before_calc'] ) && !empty( $cart_item['addons_price_before_calc'] ) ) {
                                    $value = $cart_item['addons_price_before_calc'];
                                }
                            }
                        }
                    }
                }
            }

            // Patch for "WooCommerce TM Extra Product Options" plugin by themeComplete
            if ( defined( 'THEMECOMPLETE_EPO_PLUGIN_FILE' ) ) {
                $this->unhook_price_overrides();

                foreach ( WC()->cart->get_cart() as $cart_item ) {
                    if ( $cart_item['data'] !== $product || !isset( $cart_item['tmhasepo'] ) || !$cart_item['tmhasepo'] || !isset( $cart_item['tm_epo_options_prices'] ) ) {
                        continue;
                    }

                    $addon_cost += $cart_item['tm_epo_options_prices'];
                }

                $this->hook_price_overrides();
            }

            // Patch for "Advanced Product Fields" by StudioWombat that might add additional costs.
            if ( function_exists( 'wapf' ) && isset( WC()->cart ) && !is_null( WC()->cart ) ) {
                foreach ( WC()->cart->get_cart() as $cart_item ) {
                    if ( $cart_item['data'] === $product && isset( $cart_item['wapf'] ) && is_array( $cart_item['wapf'] ) ) {
                        foreach ( $cart_item['wapf'] as $addon ) {
                            if ( isset( $addon['price'] ) && is_array( $addon['price'] ) ) {
                                foreach ( $addon['price'] as $addon_price ) {
                                    $addon_cost += $addon_price['value'];
                                }
                            }
                        }
                    }
                }
            }

            if ( !isset( $sale->use_regular_price ) || $sale->use_regular_price ) {

                $price = $product->get_regular_price();
                if ( empty( $price ) ) {
                    $price = $product->get_price();
                }

                if ( !empty( $price ) ) {
                    $value = $price;
                }
            }

            $discounted_price = $this->get_discounted_price( $sale, $value );

            if ( $discounted_price < $value ) {
                $value = $discounted_price + $addon_cost;
            }
        }

        $this->hook_price_overrides();

        return apply_filters( 'pwos_to_default_currency', $value );
    }

    function pwos_to_default_currency( $amount ) {
        // WooCommerce Currency Switcher by realmag777
        if ( isset( $GLOBALS['WOOCS'] ) ) {
            $cs = $GLOBALS['WOOCS'];
            $default_currency = false;
            $currencies = $cs->get_currencies();

            foreach ( $currencies as $currency ) {
                if ( $currency['is_etalon'] === 1 ) {
                    $default_currency = $currency;
                    break;
                }
            }

            if ( $default_currency ) {
                if ( $cs->current_currency != $default_currency['name'] ) {
                    $amount = (float) $cs->back_convert( $amount, $currencies[ $cs->current_currency ]['rate'] );
                }
            }
        }

        return $amount;
    }

    function get_post_metadata( $value, $post_id, $meta_key, $single ) {
        if ( '_flatsome_product_percentage' === $meta_key ) {
            return null;
        } else {
            return $value;
        }
    }

    function woocommerce_variation_prices( $prices_array, $product ) {
        foreach( $prices_array['price'] as $variation_id => $price ) {
            $variation = wc_get_product( $variation_id );

            $sales = $this->get_product_sales( $variation );
            foreach ( $sales as $sale ) {

                if ( !isset( $sale->use_regular_price ) || $sale->use_regular_price ) {
                    $value = $prices_array['regular_price'][ $variation_id ];
                } else {
                    $value = $price;
                }

                $discounted_price = $this->get_discounted_price( $sale, $value );
                if ( $discounted_price < $price ) {
                    $prices_array['price'][ $variation_id ] = $discounted_price;
                    $prices_array['sale_price'][ $variation_id ] = $discounted_price;
                }
            }

        }

        return $prices_array;
    }

    function get_product_sales( $product ) {
        $sales = array();

        if ( $this->wc_min_version( '3.0' ) ) {
            $parent_id = $product->get_parent_id() > 0 ? $product->get_parent_id() : $product->get_id();
            $product_id = $product->get_id();
        } else {
            if ( !empty( $product->variation_id ) ) {
                $parent_id = $product->id;
                $product_id = $product->variation_id;
            } else {
                $parent_id = !empty( $product->parent_id ) ? $product->parent_id : $product->id;;
                $product_id = $product->id;
            }
        }

        foreach ( $this->get_active_sales() as $sale ) {
            $product_included = false;

            if ( is_array( $sale->product_categories_included ) && count( $sale->product_categories_included ) > 0 ) {
                // This is inside the loop in case there are no active deals. No need to query product_cat_ids unnecessarily.
                if ( !isset( $product_categories ) ) {
                    $product_categories = wc_get_product_cat_ids( $parent_id );
                }

                // Included by category.
                if ( count( array_intersect( $product_categories, $sale->product_categories_included ) ) > 0 ) {
                    $product_included = true;
                }
            }

            if ( is_array( $sale->product_categories_excluded ) && count( $sale->product_categories_excluded ) > 0 ) {
                // This is inside the loop in case there are no active deals. No need to query product_cat_ids unnecessarily.
                if ( !isset( $product_categories ) ) {
                    $product_categories = wc_get_product_cat_ids( $parent_id );
                }

                // Included by category.
                if ( count( array_intersect( $product_categories, $sale->product_categories_excluded ) ) > 0 ) {
                    $product_included = false;
                }
            }

            if ( is_array( $sale->include_product_ids ) && ( in_array( $product_id, $sale->include_product_ids ) || in_array( $parent_id, $sale->include_product_ids ) ) ) {
                $product_included = true;
            }

            if ( true === $product_included && is_array( $sale->exclude_product_ids ) ) {
                // Ignore excluded products or variations.
                if ( in_array( $product_id, $sale->exclude_product_ids ) || in_array( $parent_id, $sale->exclude_product_ids ) ) {
                    $product_included = false;
                }
            }

            if ( true === $product_included && $sale->ignore_sale_products ) {
                $raw_sale_price = get_post_meta( $product->get_id(), '_sale_price', true );
                if ( !empty( $raw_sale_price ) ) {
                    $raw_regular_price = get_post_meta( $product->get_id(), '_regular_price', true );
                    if ( $raw_sale_price < $raw_regular_price ) {
                        $product_included = false;
                    }
                }
            }

            if ( true === $product_included ) {
                $sales[] = $sale;
            }
        }

        return $sales;
    }

    function get_discounted_price( $sale, $price ) {
        if ( empty( $price ) ) {
            return $price;
        }

        if ( $sale->deal_type == 'fixed' ) {
            return max( 0, $price - $sale->fixed_discount );
        } else {
            $decimals = apply_filters( 'wc_get_price_decimals', get_option( 'woocommerce_price_num_decimals', 2 ) );
            return round( $price * ( 1.0 - ( $sale->discount_percentage / 100 ) ), $decimals );
        }
    }

    function woocommerce_product_is_on_sale( $is_on_sale, $product ) {
        if ( count( $this->get_product_sales( $product ) ) > 0 ) {
            $is_on_sale = true;
        }

        return $is_on_sale;
    }

    public static function plugin_activate() {
        if ( ! current_user_can( 'activate_plugins' ) ) {
            return;
        }

        PW_On_Sale::register_post_types();
    }

    public static function register_post_types() {
        if ( post_type_exists('pw_on_sale') ) {
            return;
        }

        $labels = array(
            'name'                  => _x( 'PW WooCommerce On Sale', 'Post Type General Name', 'pimwick' ),
            'singular_name'         => _x( 'PW WooCommerce On Sale', 'Post Type Singular Name', 'pimwick' ),
            'menu_name'             => __( 'PW On Sale!', 'pimwick' ),
            'name_admin_bar'        => __( 'PW On Sale!', 'pimwick' ),
            'archives'              => __( 'Sale archives', 'pimwick' ),
            'parent_item_colon'     => __( 'Parent PW On Sale!:', 'pimwick' ),
            'all_items'             => __( 'All sales', 'pimwick' ),
            'add_new_item'          => __( 'Add New sale', 'pimwick' ),
            'add_new'               => __( 'Create new sale', 'pimwick' ),
            'new_item'              => __( 'New sale', 'pimwick' ),
            'edit_item'             => __( 'Edit sale', 'pimwick' ),
            'update_item'           => __( 'Update sale', 'pimwick' ),
            'view_item'             => __( 'View sale', 'pimwick' ),
            'search_items'          => __( 'Search sales', 'pimwick' ),
            'not_found'             => __( 'Not found', 'pimwick' ),
            'not_found_in_trash'    => __( 'Not found in Trash', 'pimwick' ),
            'featured_image'        => __( 'Logo', 'pimwick' ),
            'set_featured_image'    => __( 'Set Logo', 'pimwick' ),
            'remove_featured_image' => __( 'Remove Logo', 'pimwick' ),
            'use_featured_image'    => __( 'Use as Logo', 'pimwick' ),
            'insert_into_item'      => __( 'Insert into item', 'pimwick' ),
            'uploaded_to_this_item' => __( 'Uploaded to this item', 'pimwick' ),
            'items_list'            => __( 'Sale list', 'pimwick' ),
            'items_list_navigation' => __( 'Sale list navigation', 'pimwick' ),
            'filter_items_list'     => __( 'Filter sale list', 'pimwick' ),
        );

        $args = array(
            'label'                 => __( 'PW On Sale!', 'pimwick' ),
            'description'           => __( 'PW On Sale!', 'pimwick' ),
            'labels'                => $labels,
            'supports'              => array( 'title' ),
            'show_ui'               => true,
            'show_in_menu'          => false,
            'has_archive'           => true
        );

        register_post_type( 'pw_on_sale', $args );
    }

    function admin_menu() {
        if ( empty ( $GLOBALS['admin_page_hooks']['pimwick'] ) ) {
            add_menu_page(
                'PW On Sale!',
                'Pimwick Plugins',
                PW_ON_SALE_REQUIRES_PRIVILEGE,
                'pimwick',
                array( $this, 'index' ),
                plugins_url( '/assets/images/pimwick-icon-120x120.png', __FILE__ ),
                6
            );

            add_submenu_page(
                'pimwick',
                'PW On Sale!',
                'Pimwick Plugins',
                PW_ON_SALE_REQUIRES_PRIVILEGE,
                'pimwick',
                array( $this, 'index' )
            );

            remove_submenu_page( 'pimwick', 'pimwick' );
        }

        add_submenu_page(
            'pimwick',
            'PW On Sale!',
            'PW On Sale!',
            PW_ON_SALE_REQUIRES_PRIVILEGE,
            'pw-on-sale',
            array( $this, 'index' )
        );

        remove_submenu_page( 'pimwick', 'pimwick-plugins' );
        add_submenu_page(
            'pimwick',
            __( 'Pimwick Plugins', 'pimwick' ),
            __( 'Our Plugins', 'pimwick' ),
            'manage_woocommerce',
            'pimwick-plugins',
            array( $this, 'other_plugins_page' )
        );
    }

    function other_plugins_page() {
        global $pimwick_more_handled;

        if ( !$pimwick_more_handled ) {
            $pimwick_more_handled = true;
            require( 'ui/more.php' );
        }
    }

    function index() {
        if ( isset( $_POST['activate-license'] ) ) {
            $this->license->activate_license( wc_clean( $_POST['license-key'] ) );
        }

        $data = get_plugin_data( __FILE__ );
        $version = $data['Version'];

        if ( isset( $_REQUEST['sale_id'] ) ) {
            $sale_id = absint( $_REQUEST['sale_id'] );

            if ( !empty( $sale_id ) ) {
                $pwos_sale = get_post( $sale_id );
            }
        }

        require( 'ui/index.php' );
    }

    function ajax_activation() {
        $license_key = sanitize_title( $_POST['license-key'] );
        $this->license->activate_license( $license_key );

        // FAILING?
        $registration['active'] = $this->license->is_premium();
        $registration['error'] = $this->license->error;

        wp_send_json( $registration );
    }

    function ajax_check_license() {

        $registration['active'] = $this->license->is_premium();
        $registration['error'] = $this->license->error;

        wp_send_json( $registration );
    }

    function wizard_field_required( $step, $value, $error_message ) {
        if ( empty( $value ) ) {
            $this->wizard_error( $step, $error_message );
        }
    }

    function wizard_error( $step, $message ) {
        $result['complete'] = false;
        $result['step'] = $step;
        $result['message'] = $message;

        wp_send_json( $result );
    }

    function ajax_save_sale() {
        $sale_id                                = absint( $_POST['sale_id'] );
        $title                                  = stripslashes( wc_clean( $_POST['title'] ) );
        $begin_date                             = wc_clean( $_POST['begin_date'] );
        $begin_time                             = wc_clean( $_POST['begin_time'] );
        $end_date                               = wc_clean( $_POST['end_date'] );
        $end_time                               = wc_clean( $_POST['end_time'] );
        $begin_datetime                         = "$begin_date $begin_time";
        $end_datetime                           = "$end_date $end_time";

        $product_categories_included            = isset( $_POST['product_categories_included'] ) && is_array( $_POST['product_categories_included'] ) ? array_map( 'intval', $_POST['product_categories_included'] ) : array();
        $product_categories_excluded            = isset( $_POST['product_categories_excluded'] ) && is_array( $_POST['product_categories_excluded'] ) ? array_map( 'intval', $_POST['product_categories_excluded'] ) : array();
        $include_product_ids                    = isset( $_POST['include_product_ids'] ) && is_array( $_POST['include_product_ids'] ) ? array_map( 'intval', $_POST['include_product_ids'] ) : array();
        $exclude_product_ids                    = isset( $_POST['exclude_product_ids'] ) && is_array( $_POST['exclude_product_ids'] ) ? array_map( 'intval', $_POST['exclude_product_ids'] ) : array();
        $ignore_sale_products                   = $_POST['ignore_sale_products'] == 'true' ? true : false;
        $deal_type                              = $_POST['deal_type'];
        $discount_percentage                    = wc_format_decimal( $_POST['discount_percentage'] );
        $fixed_discount                         = wc_format_decimal( $_POST['fixed_discount'] );
        $use_regular_price                      = $_POST['use_regular_price'] == 'true' ? true : false;

        //
        // BEGIN VALIDATION
        //
        if ( count( $product_categories_included ) == 0 && count( $include_product_ids ) == 0 ) {
            $this->wizard_error( 1, 'Select at least 1 Product Category or 1 Included Product.', 1 );
        }
        $this->wizard_field_required( 1, $begin_date, 'Begin Date is required.' );
        $this->wizard_field_required( 1, $begin_time, 'Begin Time is required.' );
        $this->wizard_field_required( 1, $end_date, 'End Date is required.' );
        $this->wizard_field_required( 1, $end_time, 'End Time is required.' );
        if ( false === strtotime( $begin_datetime ) ) {
            $this->wizard_error( 1, "$begin_datetime is not a valid date and time." );
        }
        if ( false === strtotime( $end_datetime ) ) {
            $this->wizard_error( 1, "$end_datetime is not a valid date and time." );
        }
        if ( strtotime( $begin_datetime ) >= strtotime( $end_datetime ) ) {
            $this->wizard_error( 1, "Begin date must come before End date." );
        }
        $this->wizard_field_required( 3, $discount_percentage, 'Discount Percentage is required.' );
        $this->wizard_field_required( 4, $title, 'Title is required.' );
        //
        // END VALIDATION
        //

        if ( empty( $sale_id ) ) {
            $sale = array();
            $sale['post_type'] = 'pw_on_sale';
            $sale['post_status'] = 'publish';
            $sale['post_title'] = $title;

            $sale_id = wp_insert_post( $sale );
        } else {
            $sale = get_post( $sale_id );
            $sale->post_title = $title;
            wp_update_post( $sale );
        }

        if ( !is_wp_error( $sale_id ) ) {
            update_post_meta( $sale_id, 'begin_date', $begin_date );
            update_post_meta( $sale_id, 'begin_time', $begin_time );
            update_post_meta( $sale_id, 'end_date', $end_date );
            update_post_meta( $sale_id, 'end_time', $end_time );
            update_post_meta( $sale_id, 'product_categories_included', $product_categories_included );
            update_post_meta( $sale_id, 'product_categories_excluded', $product_categories_excluded );
            update_post_meta( $sale_id, 'include_product_ids', $include_product_ids );
            update_post_meta( $sale_id, 'exclude_product_ids', $exclude_product_ids );
            update_post_meta( $sale_id, 'ignore_sale_products', $ignore_sale_products );
            update_post_meta( $sale_id, 'deal_type', $deal_type );
            update_post_meta( $sale_id, 'discount_percentage', $discount_percentage );
            update_post_meta( $sale_id, 'fixed_discount', $fixed_discount );
            update_post_meta( $sale_id, 'use_regular_price', $use_regular_price );

            $result['complete'] = true;
            wp_send_json( $result );

        } else {
            $this->wizard_error( 1, $sale_id->get_error_message() );
        }
    }

    function ajax_sale_delete() {
        $sale_id = absint( $_POST['sale_id'] );
        wp_delete_post( $sale_id, true );
        wp_die();
    }

    function navigation_buttons( $step ) {
        global $pwos_step;

        $pwos_step = $step;

        require( 'ui/wizard/navigation_buttons.php' );
    }

    function admin_enqueue_scripts( $hook ) {
        if ( !empty( $hook ) && substr( $hook, -strlen( 'pw-on-sale' ) ) === 'pw-on-sale' ) {
            wp_register_style( 'jquery-ui-style', $this->relative_url( '/assets/css/jquery-ui-style.min.css' ), array(), PW_ON_SALE_VERSION );
            wp_enqueue_style( 'jquery-ui-style' );

            wp_enqueue_script( 'jquery-ui-datepicker' );

            wp_enqueue_script( 'wc-admin-meta-boxes' );
            wp_enqueue_style( 'woocommerce_admin_styles' );

            wp_register_style( 'pw-on-sale-style', $this->relative_url( '/assets/css/style.css' ), array( 'woocommerce_admin_styles' ), PW_ON_SALE_VERSION );
            wp_enqueue_style( 'pw-on-sale-style' );

            wp_register_style( 'pw-on-sale-font-awesome', $this->relative_url( '/assets/css/font-awesome.min.css' ), array(), PW_ON_SALE_VERSION );
            wp_enqueue_style( 'pw-on-sale-font-awesome' );

            wp_enqueue_script( 'pw-on-sale-script', $this->relative_url( '/assets/js/script.js' ), array( 'wc-admin-meta-boxes', 'jquery' ), PW_ON_SALE_VERSION );
        }

        wp_register_style( 'pw-on-sale-icon', plugins_url( '/assets/css/icon-style.css', __FILE__ ), array(), PW_ON_SALE_VERSION );
        wp_enqueue_style( 'pw-on-sale-icon' );
    }

    function relative_url( $url ) {
        return plugins_url( $url, __FILE__ );
    }

    function require_file( $filename, $once = false ) {
        $relative_path = trailingslashit( __DIR__ ) . ltrim( $filename, '/' );
        if ( $once === true ) {
            require_once( $relative_path );
        } else {
            require( $relative_path );
        }
    }

    function wc_min_version( $version ) {
        return version_compare( WC()->version, $version, ">=" );
    }

    function starts_with( $needle, $haystack ) {
        $length = strlen( $needle );
        return ( substr( $haystack, 0, $length ) === $needle );
    }

    /**
     * Source: http://wordpress.stackexchange.com/questions/14652/how-to-show-a-hierarchical-terms-list
     * Recursively sort an array of taxonomy terms hierarchically. Child categories will be
     * placed under a 'children' member of their parent term.
     * @param Array   $cats     taxonomy term objects to sort
     * @param Array   $into     result array to put them in
     * @param integer $parentId the current parent ID to put them in
     */
    function sort_terms_hierarchicaly(array &$cats, array &$into, $parentId = 0) {
        foreach ( $cats as $i => $cat ) {
            if ( $cat->parent == $parentId ) {
                $into[$cat->term_id] = $cat;
                unset( $cats[$i] );
            }
        }

        foreach ( $into as $topCat ) {
            $topCat->children = array();
            $this->sort_terms_hierarchicaly( $cats, $topCat->children, $topCat->term_id );
        }
    }

    function hierarchical_select($categories, $selected_category_ids, $level = 0, $parent = NULL, $prefix = '') {
        foreach ( $categories as $category ) {
            $selected = selected( in_array( $category->term_id, $selected_category_ids ), true, false );
            echo "<option value='" . esc_attr( $category->term_id ) . "' $selected>$prefix " . esc_html( $category->name ) . "</option>\n";

            if ( $category->parent == $parent ) {
                $level = 0;
            }

            if ( count( $category->children ) > 0 ) {
                echo $this->hierarchical_select( $category->children, $selected_category_ids, ( $level + 1 ), $category->parent, "$prefix " . esc_html( $category->name ) . " &#8594;" );
            }
        }
    }
}

register_activation_hook( __FILE__, array( 'PW_On_Sale', 'plugin_activate' ) );

global $pw_on_sale;
$pw_on_sale = new PW_On_Sale();

endif;
